// RingWorm v 1.2 (c) 1998 Graham Mitchell <mitchell@onr.com> - 19 Oct 1998

// main program

/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License (LICENSE.TXT) for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "const.h"
#include "worm.cc"
#include "item.cc"
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <iostream.h>
#include <allegro.h>

void init();
void process_flags(int, char**);
void ticker(...);
bool collide(int);
void consume(int,int);
void get_safe_loc( Point& trial, int & rad );
bool is_taken ( Point trial, int r );
void display_fps();
void display_score();

volatile int last_fps = 0;
volatile int frames = 0;
Worm* worms = NULL;
Item* food = NULL;
Point food_loc;
int food_rad;

int main(int argc, char* argv[])
{
	int turbo = 0;

	// Find number of "players" and other command line parameters
	if ( argc > 1 )
		process_flags(argc, argv);

	worms = new Worm[PLAYERS];
	food = new Item[PIZZAS];

	allegro_init();
	init();

	GAME_QUIT = false;

	// MAIN GAME LOOP
	do 
	{
		// check all the worms
		for ( int i=0; i<PLAYERS && !GAME_QUIT; ++i )
		{
			// draw everybody's current location
			worms[i].draw();

			// check if anyone has run into food
			if ( USE_FOOD )
				for ( int j=0; j<PIZZAS; ++j )
				if ( food[j].touches(worms[i].location(),worms[i].radius()) )
					consume(i,j);

			// update my current location
			GAME_QUIT = ( GAME_QUIT || worms[i].update() );

			// check for collisions
			if ( CHECK_COLLISIONS )
				GAME_QUIT = ( GAME_QUIT || collide(i) );
		}
		if ( SHOW_SCORE )
			display_score();
		else if ( SHOW_FPS )
		{
			display_fps();
			++frames;
		}
		if ( BENCHMARK )
			GAME_QUIT = false;
		else
		{
			if ( turbo == 0 )
				vsync();
			turbo = (turbo+1)%TURBO;
		}

		GAME_QUIT = ( GAME_QUIT || key[KEY_ESC] );

	} while ( ! GAME_QUIT );

	allegro_exit();
	cout << "Score: " << worms[0].score() << endl;
	cout << endl;
	cout << "RingWorm v 1.2 (c) 1998 Graham Mitchell - 19 Oct 1998\n";
	cout << "http://www.onr.com/user/mitchell/\n\n";

	return 0;
}


void process_flags(int argc, char* argv[])
{
	for ( int i=1; i<argc; ++i )
	{
		if ( argv[i][1] >= '0' && argv[i][1] <= '9' )
			// check if this is our # of worms argument
			PLAYERS = atoi(&argv[i][1]);
		else if ( stricmp(&argv[i][1],"ghost") == 0 )
			// or maybe the GHOST parameter
			CHECK_COLLISIONS = false;
		else if ( stricmp(&argv[i][1],"bench") == 0 )
		{
			// or perhaps the BENCH parameter
			BENCHMARK = true;
			SHOW_FPS = true;
			USE_FOOD = false;
		}
		else if ( strnicmp(&argv[i][1],"turbo:",6) == 0 )
			TURBO = ( argv[i][7] - '0' );
		else if ( strnicmp(&argv[i][1],"food:",5) == 0 )
			PIZZAS = ( argv[i][6] - '0' );
		else if ( stricmp(&argv[i][1],"bounce") == 0 )
		{	BOUNCE = true;	WRAP = false;	}
		else if ( stricmp(&argv[i][1],"wrap") == 0 )
		{	WRAP = true;	BOUNCE = false;	}
	}
	SHOW_SCORE = ( PLAYERS == 1 && ! SHOW_FPS );

}


void display_fps()
{
	char buf[8];
	int wid;
	sprintf(buf, "%d", last_fps);
	wid = strlen(buf);
	rectfill(screen, WIDTH-wid*8, 0, WIDTH, 9, BLANK );
	textout(screen, font, buf, WIDTH-wid*8, 1, 0);
}


void display_score()
{
	char buf[8];
	int wid;
	sprintf(buf, "%d", worms[0].score());
	wid = strlen(buf);
	rectfill(screen, WIDTH-wid*8, 0, WIDTH, 9, BLANK );
	textout(screen, font, buf, WIDTH-wid*8, 1, 0);
}


bool collide(int i)
{
	// determine if this worm is touching anybody else

	for ( int j=0; j<PLAYERS; ++j )
		if ( worms[j].touches(worms[i].location(),1) )
			return true;

	return false;
}

void consume(int i,int j)
{
	// someone ran into some food.  Update their score and size

	worms[i].award( food[j].radius() );
	worms[i].grow( 2*food[j].radius() );

	food[j].erase();
	get_safe_loc( food_loc, food_rad );
	food[j].init(food_loc, food_rad, RED);
	food[j].draw();
}


void ticker(...)
{
	last_fps = frames;
	frames = 0;
}
END_OF_FUNCTION (ticker);


void init()
{
	int r = RADIUS;
	int s = SPEED;

	install_keyboard();
	install_timer();

	LOCK_FUNCTION (ticker);
	LOCK_VARIABLE (frames);
	LOCK_VARIABLE (last_fps);

	install_int(ticker, 1000);

	set_gfx_mode(GFX_AUTODETECT, WIDTH, HEIGHT, 0, 0);
	clear_to_color(screen, BLANK);
	text_mode(-1);

	table_init();

	// Initialize players

	for ( int i=0; i<PLAYERS; ++i )
	{
		if ( i%4 == 0 )
			worms[i].init(i,WIDTH/2+3*i,HEIGHT-r,r,PURPLE,KEY_LEFT,KEY_RIGHT,s,ANG_N);
		else if ( i%4 == 1 )
			worms[i].init(i,WIDTH/2-3*i,2,r,GREEN,KEY_Z,KEY_C,s,ANG_S);
		else if ( i%4 == 2 )
			worms[i].init(i,r,HEIGHT/2+3*i,r,RED,KEY_Q,KEY_E,s,ANG_E);
		else if ( i%4 == 3 )
			worms[i].init(i,WIDTH-r,HEIGHT/2-3*i,r,GREY,KEY_J,KEY_L,s,ANG_W);
	}

	if ( USE_FOOD )
	{
		// Start up random number generator

		srand ( unsigned( time(NULL) ) );

		// Place the 'pizza'

		for ( int i=0; i<PIZZAS; ++i )
		{
			get_safe_loc(food_loc, r);
			food[i].init(food_loc, r, RED);
			food[i].draw();
		}
	}

}

void get_safe_loc( Point& trial, int & rad )
{

	do {
		rad = rand()%PIZZA_VAR + PIZZA_MIN;
		trial.x = rand()%(WIDTH-2*rad) + rad;
		trial.y = rand()%(HEIGHT-2*rad) + rad;
		if ( key[KEY_ESC] )
			break;

	} while ( is_taken ( trial, rad ) );

}


bool is_taken ( Point trial, int r )
{
	int j;
	for ( j=0; j<PIZZAS; ++j )
		if ( food[j].touches(trial,r) )
			return true;
	for ( j=0; j<PLAYERS; ++j )
		if ( worms[j].touches(trial,r) )
			return true;

	return false;
}

