// RingWorm v 1.2 (c) 1998 Graham Mitchell <mitchell@onr.com> - 19 Oct 1998

// Queue class, modified with subscript operator

/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License (LICENSE.TXT) for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __QUEUE_H
#define __QUEUE_H

const int QUEUE_INITIAL_SIZE = 30;

template <class Type>
class Queue
{	

public:

	// constructors/destructor

	Queue();
	Queue(int);
	~Queue();

	// accessor functions

	Type deque();
	void enque(Type);
	Type operator [] (int) const;

	int depth() { return items; }
	bool isEmpty() { return ( items == 0 ); }

private:

	void grow();	// increase in size when queue fills

	int front;
	int back;
	int items;

	Type* array;
	int size;

};
#endif
/* *********************************************************************
                           Constructors
   *********************************************************************/

template <class Type>
Queue<Type>::Queue()
{
	size = QUEUE_INITIAL_SIZE;
	items = 0;
	array = new Type[size];
	front = 1;
	back = 0;
}


template <class Type>
Queue<Type>::Queue(int numSlots)
{
	size = numSlots;
	items = 0;
	array = new Type[size];
	front = 1;
	back = 0;
}


template <class Type>
Queue<Type>::~Queue()
{
	delete [] array;
}

/*************************************************************************
                           Accessors
 *************************************************************************/

template <class Type>
void Queue<Type>::enque(Type item)
{
	if ( items+1 >= size )
		grow();
	++items;
	back = (back + 1) % size;
	array[back] = item;
}


template <class Type>
Type Queue<Type>::deque()
{
	Type chosen = array[front];
	front = (front + 1) % size;
	--items;
	return chosen;
}


template <class Type>
Type Queue<Type>::operator [] (int pos) const
{
	// oldest item in the queue will be at subscript 0,
	// newest item in the queue will be at subscript 'items'-1

	return ( array[ (pos+front)%size ] );
}


template <class Type>
void Queue<Type>::grow()
{
	// we've run out of room; make the array larger by 50%

	int i, newsize = int(size*1.5);
	Type* buf = new Type[newsize];

	// now copy everything over, being sure to wrap

	for ( i=0; i<items; ++i )
	{
		buf[i] = array[front];
		front = (front + 1) % size;
	}

	// and now update all the values to reference the new array

	delete [] array;
	array = buf;
	front = 0;
	back = items-1;
	size = newsize;
}

